#!/usr/bin/env python
# coding: utf-8

# In[85]:


def find_circle_circle_intersections(center0, radius0, center1, radius1):
    '''Find the 0, 1, or 2 points where the circles intersect.'''
    # Find the distance between the centers.
    dx = center0[0] - center1[0]
    dy = center0[1] - center1[1]
    dist = math.sqrt(dx * dx + dy * dy)

    # See if the circles are so far apart they don't intersect.
    if dist > radius0 + radius1:
        print('Circles too far apart, no intersections')
        return ()

    # See if one circle contains the other.
    if dist < abs(radius0 - radius1):
        print('One circle inside the other, no intersections')
        return ()

    # See if the circles coincide.
    if math.isclose(dist, 0) and math.isclose(radius0, radius1):
        print('Circles coincide, no intersections')
        return ()

    # Otherwise we have 1 or 2 intersections.
    # Find a and h.
    a = (radius0 * radius0 - radius1 * radius1 + dist * dist) / (2 * dist)
    h = math.sqrt(radius0 * radius0 - a * a)

    # Find P2.
    cx2 = center0[0] + a * (center1[0] - center0[0]) / dist
    cy2 = center0[1] + a * (center1[1] - center0[1]) / dist

    # Get the points P3.
    intersection1 = (
        cx2 + h * (center1[1] - center0[1]) / dist,
        cy2 - h * (center1[0] - center0[0]) / dist)
    intersection2 = (
        cx2 - h * (center1[1] - center0[1]) / dist,
        cy2 + h * (center1[0] - center0[0]) / dist)

    # See if we have 1 or 2 solutions.
    print(f'{intersection1 = }')
    print(f'{intersection2 = }')
    if math.isclose(intersection1[0], intersection2[0]) and \
            math.isclose(intersection1[1], intersection2[1]):
        # One intersection. They're the same point.
        return (intersection1,)
    return (intersection1, intersection2)


# In[86]:


import tkinter as tk
from PIL import Image, ImageTk
import math

# Layout constants.
MARGIN = 5
WINDOW_WID = 400
WINDOW_HGT = 300

class CircleCircleApp:
    # Create and manage the tkinter interface.
    def __init__(self):
        # Make the main interface.
        self.window = tk.Tk()
        self.window.title('circle_circle_intersections')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry(f'{WINDOW_WID}x{WINDOW_HGT}')

        # Build the menus.
        self.build_menus()

        # Build the rest of the UI here...
        self.canvas = tk.Canvas(self.window, bg='white', cursor='cross',
            borderwidth=2, relief=tk.SUNKEN)
        self.canvas.pack(padx=10, pady=(0, 10),
            side=tk.BOTTOM, fill=tk.BOTH, expand=True)

        # Initially we are not drawing a circle.
        self.start_point = None
        self.end_point = None
        self.current_circle = None

        # Make a list to hold two circles.
        # Each circle looks like ((cx, cy), radius).
        self.circles = []

        # Track Button-1 down, Button-1 motion, and Button-1 up.
        self.canvas.bind("<Button-1>", self.mouse_down)
        self.canvas.bind("<B1-Motion>", self.mouse_move)
        self.canvas.bind("<ButtonRelease-1>", self.mouse_up)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)

        self.build_file_menu(menu_bar)

    def build_file_menu(self, menu_bar):
        '''Build the File menu.'''
        file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=file_menu)

        # Exit
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')
        file_menu.add_command(label='Exit', compound='left',
            command=self.mnu_exit, image=self.exit_photo_image)

    def mnu_exit(self):
        self.kill_callback()

    def kill_callback(self):
        self.window.destroy()

    def mnu_clear(self):
        self.canvas.delete(tk.ALL)

    def mouse_down(self, event):
        '''Record the circle's starting point.'''
        self.start_point = (event.x, event.y)

        # If we already have two circles, start over.
        if len(self.circles) == 2:
            self.circles = []
            self.canvas.delete(tk.ALL)

    def mouse_move(self, event):
        # Ignore extraneous events.
        if self.start_point is None: return

        # If there is a current new circle, remove it.
        if self.current_circle is not None:
            self.canvas.delete(self.current_circle)

        # Create the new temporary circle.
        self.end_point = self.get_circle_endpoint(self.start_point, (event.x, event.y))
        self.current_circle = self.canvas.create_oval(
            self.start_point, self.end_point, outline='red')

    def mouse_up(self, event):
        # Ignore extraneous events.
        if self.start_point is None: return

        # If there is a current new circle, remove it.
        if self.current_circle is not None:
            self.canvas.delete(self.current_circle)

        # Create the permanent circle.
        self.end_point = self.get_circle_endpoint(self.start_point, (event.x, event.y))
        self.canvas.create_oval(
            self.start_point, self.end_point, outline='green')

        # Save the new circle's data.
        cx = (self.start_point[0] + self.end_point[0]) / 2
        cy = (self.start_point[1] + self.end_point[1]) / 2
        radius = abs(self.start_point[0] - self.end_point[0]) / 2
        self.circles.append(((cx, cy), radius))

        # If we now have two circles, find their intersections.
        if len(self.circles) == 2:
            intersections = find_circle_circle_intersections(
                center0=self.circles[0][0],
                radius0=self.circles[0][1],
                center1=self.circles[1][0],
                radius1=self.circles[1][1])
            for intersection in intersections:
                POINT_RADIUS = 3
                self.canvas.create_oval(
                    intersection[0] - POINT_RADIUS,
                    intersection[1] - POINT_RADIUS,
                    intersection[0] + POINT_RADIUS,
                    intersection[1] + POINT_RADIUS,
                    fill='red')
            if len(intersections) > 0:
                print(f'{len(intersections)} intersections:')
                print(f'    {intersections}')

        # We are no longer creating a circle.
        self.start_point = None
        self.end_point = None
        self.current_circle = None

    def get_circle_endpoint(self, start_point, end_point):
        '''Return the other corner for the circle.'''
        # Get the width and height.
        dx = end_point[0] - self.start_point[0]
        dy = end_point[1] - self.start_point[1]

        # Use the larger of the width and height.
        if abs(dx) < abs(dy):
            # Make dx have the size of dy but in the direction of dx.
            dx = math.copysign(dy, dx)
        else:
            # Make dy have the size of dx but in the direction of dy.
            dy = math.copysign(dx, dy)
        return (start_point[0] + dx, start_point[1] + dy)


# In[87]:


CircleCircleApp()
print('Done')


# In[ ]:




